CREATE OR REPLACE FUNCTION HORAS_CUMPRIPRAZO_SOL
   (P_CDFILIAL   IN ITEMPRODUTO.CDFILIAL%TYPE,
    P_NRSOLISERV  IN SOLISERV.NRSOLISERV%TYPE) RETURN NUMBER
AS
   V_TOTAPLAN    PLANATIVTARE.QTTEMPLAITAR%TYPE;
   V_TEMPOREAL   PLANATIVTARE.QTTEMPLAITAR%TYPE;
   V_PERCESFORCO PLANATIVTARE.QTTEMPLAITAR%TYPE;

   V_TOTAATRA    ITEMPROIMPOS.VRBASECALCIT%TYPE;
   V_DATAFIM     NOTAFISCAL.DTENTRSAID%TYPE;
   V_DATAINI     NOTAFISCAL.DTENTRSAID%TYPE;
   V_DTMARCOSOL  NOTAFISCAL.DTENTRSAID%TYPE;
   V_NCONC       ITEMPROIMPOS.VRBASECALCIT%TYPE;
   V_DTMAXATIV   NOTAFISCAL.DTENTRSAID%TYPE;
   V_QTDEATIV    ITEMPROIMPOS.VRBASECALCIT%TYPE;
   V_TEMPOPLA    ITEMPROIMPOS.VRBASECALCIT%TYPE;
BEGIN
   V_DATAFIM := SYSDATE;

   V_TOTAATRA := 0;

   /* quantidade de atividades finalizadas da solicitao */
   SELECT COUNT(*)
   INTO V_TOTAPLAN
   FROM TAREFA T, PLANATIVTARE P
   WHERE T.CDFILIAL   = P_CDFILIAL   AND
         T.NRSOLISERV = P_NRSOLISERV AND
         T.CDFILIAL   = P.CDFILIAL   AND
         T.NRTAREFA   = P.NRTAREFA   AND
         TRUNC(DTFIPATIVTAR) < V_DATAFIM;

   /*Tempo total planejado para a solicitao*/
   SELECT SUM(P.QTTEMPLAITAR)
   INTO V_TEMPOPLA
   FROM  TAREFA T, PLANATIVTARE P
   WHERE T.CDFILIAL   = P_CDFILIAL   AND
         T.NRSOLISERV = P_NRSOLISERV AND
         T.CDFILIAL   = P.CDFILIAL   AND
         T.NRTAREFA   = P.NRTAREFA;

   /*Tempo apropriado da solicitao*/
   SELECT ROUND(SUM(DTHFAPATIVTA - DTHIAPATIVTA)*24,2)
   INTO V_TEMPOREAL
   FROM  TAREFA T, APROATIVTARE A
   WHERE T.CDFILIAL   = P_CDFILIAL   AND
         T.NRSOLISERV = P_NRSOLISERV AND
         T.CDFILIAL   = A.CDFILIAL   AND
         T.NRTAREFA   = A.NRTAREFA;

   /*data da soli*/
   SELECT SO.DTCOMPROMISS
     INTO V_DTMARCOSOL
     FROM SOLISERV SO
    WHERE SO.CDFILIAL     = P_CDFILIAL
      AND SO.NRSOLISERV   = P_NRSOLISERV;

   /*quantidade de atividades no concludas e no finalizadas */
   SELECT COUNT(*)
     INTO V_NCONC
     FROM TAREFA TA, PLANATIVTARE PL
    WHERE TA.CDFILIAL   = PL.CDFILIAL
      AND TA.NRTAREFA   = PL.NRTAREFA
      AND TA.CDFILIAL   = P_CDFILIAL
      AND TA.NRSOLISERV = P_NRSOLISERV
      AND PL.IDSITATIVTAR <> 'C'
      AND PL.DTFIPATIVTAR IS NOT NULL;

   IF V_NCONC = 0 THEN
      BEGIN
      /*maior data de concluso(finalizao) das atividades da solicitao*/
         SELECT MAX(PL.DTCONATIVTAR)
           INTO V_DTMAXATIV
           FROM TAREFA TA, PLANATIVTARE PL
          WHERE TA.CDFILIAL = PL.CDFILIAL
            AND TA.NRTAREFA = PL.NRTAREFA
            AND TA.CDFILIAL = P_CDFILIAL
            AND TA.NRSOLISERV = P_NRSOLISERV
            AND PL.DTFIPATIVTAR IS NOT NULL;
         EXCEPTION
            WHEN NO_DATA_FOUND THEN
               NULL;
         END;
                  /* verifica se as atividades estao atrasadas*/
         IF V_DTMAXATIV > V_DTMARCOSOL THEN
            /*quantidade de de atividades finalizadas */
            SELECT COUNT(*)
            INTO V_QTDEATIV
            FROM TAREFA TA, PLANATIVTARE PL
            WHERE TA.CDFILIAL = PL.CDFILIAL
              AND TA.NRTAREFA = PL.NRTAREFA
              AND TA.CDFILIAL = P_CDFILIAL
              AND TA.NRSOLISERV = P_NRSOLISERV
              AND PL.DTFIPATIVTAR IS NOT NULL;

            V_TOTAATRA := V_QTDEATIV;
   
         END IF;
   ELSE
         SELECT COUNT(*)
         INTO V_QTDEATIV
         FROM  TAREFA T, PLANATIVTARE P
         WHERE T.CDFILIAL = P_CDFILIAL
           AND T.NRSOLISERV = P_NRSOLISERV
           AND T.CDFILIAL   = P.CDFILIAL
           AND T.NRTAREFA   = P.NRTAREFA
           AND TRUNC(DTFIPATIVTAR) < V_DATAFIM
           AND ((P.IDSITATIVTAR = 'C' AND TRUNC(NVL(DTFIRATIVTAR,DTCONATIVTAR)) > V_DATAFIM)
            OR  (IDSITATIVTAR <> 'C' AND TRUNC(DTFIRATIVTAR) > V_DATAFIM)
            OR  (IDSITATIVTAR <> 'C' AND DTFIRATIVTAR IS NULL));

         V_TOTAATRA := V_QTDEATIV;

   END IF;
   V_PERCESFORCO := 0;
   IF V_TEMPOPLA <> 0 THEN
      V_PERCESFORCO := (V_TEMPOREAL / V_TEMPOPLA);
      IF V_PERCESFORCO > 1 THEN
         V_PERCESFORCO := 1;
      END IF;
   END IF;

   IF V_TOTAPLAN = 0 THEN
      RETURN(0);
   ELSE
      IF ROUND(((V_TOTAATRA/V_TOTAPLAN) * 100) * V_PERCESFORCO,0) > 100 THEN
         RETURN(100);
      ELSE
         RETURN(ROUND(((V_TOTAATRA/V_TOTAPLAN) * 100) * V_PERCESFORCO,0));
      END IF;
   END IF;  
END;
/
-- VERSAO --
BEGIN ATUVEROBJ('HORAS_CUMPRIPRAZO_SOL','5.01.033'); END;
/

